@@ -20,8 +20,8 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
-#ifndef ABSTRACTABSTRACTCONTEXTBUILDER_H
-#define ABSTRACTABSTRACTCONTEXTBUILDER_H
+#ifndef KDEVPLATFORM_ABSTRACTABSTRACTCONTEXTBUILDER_H
+#define KDEVPLATFORM_ABSTRACTABSTRACTCONTEXTBUILDER_H
 
 #include <climits>
 
@@ -277,15 +277,13 @@ protected:
   inline void setCompilingContexts(bool compilingContexts) { m_compilingContexts = compilingContexts; }
 
   /**
-   * Create child contexts for only a portion of the document at \a url.
+   * Create child contexts for only a portion of the document.
    *
-   * \param url The url of the document to parse
    * \param node The AST node which corresponds to the context to parse
    * \param parent The DUContext which encompasses the \a node.
    * \returns The DUContext which was reparsed, ie. \a parent.
    */
-  DUContext* buildSubContexts( const KUrl& url, T *node,
-                                         DUContext* parent )
+  DUContext* buildSubContexts( T *node, DUContext* parent )
   {
   //     m_compilingContexts = true;
   //     m_recompiling = false;
@@ -569,6 +567,7 @@ protected:
    * \param identifier The identifier which corresponds to the context.
    * \returns the opened context.
    */
+  
   virtual DUContext* openContextInternal( const RangeInRevision& range, DUContext::ContextType type, const QualifiedIdentifier& identifier )
   {
     Q_ASSERT( m_compilingContexts );
@@ -585,9 +584,16 @@ protected:
         for ( ; currentIndex < childContexts.count(); ++currentIndex )
         {
           DUContext* child = childContexts.at( currentIndex );
+          RangeInRevision childRange = child->range();
 
-          //For unnamed child-ranges, we still do range-comparison, because we cannot distinguish them in other ways
-          if ( child->type() == type && child->localScopeIdentifier() == identifier && (!identifier.isEmpty() || child->range() == range) )
+          if (child->type() != type) {
+            continue;
+          }
+          // We cannot update a contexts local scope identifier, that will break many other parts, like e.g.
+          // the CodeModel of child contexts or declarations.
+          // For unnamed child-ranges, we still do range-comparison, because we cannot distinguish them in other ways
+          if ((!identifier.isEmpty() && child->localScopeIdentifier() == identifier)
+              || (identifier.isEmpty() && child->localScopeIdentifier().isEmpty() && !childRange.isEmpty() && childRange == range))
           {
             // Match
             ret = child;
@@ -597,14 +603,11 @@ protected:
             ret->clearImportedParentContexts();
             ++currentIndex;
             break;
-          }else{
-            break;
           }
         }
         if(ret)
           nextContextIndex() = currentIndex; //If we had a match, jump forward to that position
-        else
-          ++nextContextIndex();   //If we did not have a match, just increment by 1
+          ///@todo We should also somehow make sure we don't get quadratic worst-case effort while updating.
       }
 
       if ( !ret )
@@ -619,6 +622,8 @@ protected:
 
         setInSymbolTable( ret );
       }else{
+        DUChainWriteLocker writeLock( DUChain::lock() );
+        Q_ASSERT(ret->localScopeIdentifier() == identifier);
         if(ret->parentContext())
           ret->setRange( range );
       }
