@@ -70,7 +70,8 @@ K_PLUGIN_FACTORY(AppWizardFactory, registerPlugin<AppWizardPlugin>();)
 K_EXPORT_PLUGIN(AppWizardFactory(KAboutData("kdevappwizard","kdevappwizard", ki18n("Project Wizard"), "0.1", ki18n("Support for creating and importing projects"), KAboutData::License_GPL)))
 
 AppWizardPlugin::AppWizardPlugin(QObject *parent, const QVariantList &)
-    :KDevelop::IPlugin(AppWizardFactory::componentData(), parent)
+    : KDevelop::IPlugin(AppWizardFactory::componentData(), parent)
+    , m_templatesModel(0)
 {
     KDEV_USE_EXTENSION_INTERFACE(KDevelop::ITemplateProvider);
     setXMLFile("kdevappwizard.rc");
@@ -80,12 +81,9 @@ AppWizardPlugin::AppWizardPlugin(QObject *parent, const QVariantList &)
     m_newFromTemplate->setText(i18n("New From Template..."));
     connect(m_newFromTemplate, SIGNAL(triggered(bool)), this, SLOT(slotNewProject()));
     m_newFromTemplate->setToolTip( i18n("Generate a new project from a template") );
-    m_newFromTemplate->setWhatsThis( i18n("<b>New project</b><p>"
-                               "This starts KDevelop's application wizard. "
-                               "It helps you to generate a skeleton for your "
-                               "application from a set of templates.</p>") );
-
-    m_templatesModel = new ProjectTemplatesModel(this);
+    m_newFromTemplate->setWhatsThis( i18n("This starts KDevelop's application wizard. "
+                                          "It helps you to generate a skeleton for your "
+                                          "application from a set of templates.") );
 }
 
 AppWizardPlugin::~AppWizardPlugin()
@@ -94,7 +92,7 @@ AppWizardPlugin::~AppWizardPlugin()
 
 void AppWizardPlugin::slotNewProject()
 {
-    m_templatesModel->refresh();
+    model()->refresh();
     AppWizardDialog dlg(core()->pluginController(), m_templatesModel);
 
     if (dlg.exec() == QDialog::Accepted)
@@ -459,11 +457,18 @@ KDevelop::ContextMenuExtension AppWizardPlugin::contextMenuExtension(KDevelop::C
     return ext;
 }
 
-QAbstractItemModel* AppWizardPlugin::templatesModel() const
+ProjectTemplatesModel* AppWizardPlugin::model()
 {
+    if(!m_templatesModel)
+        m_templatesModel = new ProjectTemplatesModel(this);
     return m_templatesModel;
 }
 
+QAbstractItemModel* AppWizardPlugin::templatesModel()
+{
+    return model();
+}
+
 QString AppWizardPlugin::knsConfigurationFile() const
 {
     return "kdevappwizard.knsrc";
@@ -490,12 +495,12 @@ QString AppWizardPlugin::name() const
 
 void AppWizardPlugin::loadTemplate(const QString& fileName)
 {
-    m_templatesModel->loadTemplateFile(fileName);
+    model()->loadTemplateFile(fileName);
 }
 
 void AppWizardPlugin::reload()
 {
-    m_templatesModel->refresh();
+    model()->refresh();
 }
 
 
