@@ -11,8 +11,8 @@
 *                                                                         *
 ***************************************************************************/
 
-#ifndef GREPDIALOG_H
-#define GREPDIALOG_H
+#ifndef KDEVPLATFORM_PLUGIN_GREPDIALOG_H
+#define KDEVPLATFORM_PLUGIN_GREPDIALOG_H
 
 #include <kdialog.h>
 #include "ui_grepwidget.h"
@@ -34,7 +34,6 @@ public:
     ~GrepDialog();
 
     void setPattern(const QString &pattern);
-    void setDirectory(const QString &dir);
     void setEnableProjectBox(bool enable);
 
     QString patternString() const;
@@ -45,12 +44,18 @@ public:
 
     bool useProjectFilesFlag() const;
     bool regexpFlag() const;
-    bool recursiveFlag() const;
     bool caseSensitiveFlag() const;
 
+    int depthValue() const;
+
     void start();
     
+public Q_SLOTS:
+    ///Sets directory(ies)/files to search in. Also it can be semicolon separated list of directories/files or one of special strings: allOpenFilesString, allOpenProjectsString
+    void setSearchLocations(const QString &dir);
+
 private Q_SLOTS:
+
     void performAction(KDialog::ButtonCode button);
     void templateTypeComboActivated(int);
     void patternComboEditTextChanged( const QString& );
@@ -60,6 +65,9 @@ private Q_SLOTS:
     void addStringToMenu(QMenu* ret, QString string);
     void synchronizeDirActionTriggered(bool);
 
+    ///Opens the dialog to select a directory to search in, and inserts it into Location(s) field.
+    void selectDirectoryDialog();
+
 private:
     // Returns the chosen directories or files (only the top directories, not subfiles)
     QList< KUrl > getDirectoryChoice() const;
