@@ -66,6 +66,7 @@
 #include "projectitemquickopen.h"
 #include "declarationlistquickopen.h"
 #include "customlistquickopen.h"
+#include "documentationquickopenprovider.h"
 #include <language/duchain/functiondefinition.h>
 #include <qmenu.h>
 #include <qdesktopwidget.h>
@@ -93,7 +94,10 @@ class QuickOpenWidgetCreator {
 
 class StandardQuickOpenWidgetCreator : public QuickOpenWidgetCreator {
   public:
-    StandardQuickOpenWidgetCreator(QStringList items, QStringList scopes) : m_items(items), m_scopes(scopes) {
+    StandardQuickOpenWidgetCreator(const QStringList& items, const QStringList& scopes)
+      : m_items(items)
+      , m_scopes(scopes)
+    {
     }
 
     virtual QString objectNameForLine() {
@@ -165,7 +169,7 @@ public:
 };
 
 K_PLUGIN_FACTORY(KDevQuickOpenFactory, registerPlugin<QuickOpenPlugin>(); )
-K_EXPORT_PLUGIN(KDevQuickOpenFactory(KAboutData("kdevquickopen","kdevquickopen", ki18n("Quick Open"), "0.1", ki18n("Quickly open resources such as files, classes and methods."), KAboutData::License_GPL)))
+K_EXPORT_PLUGIN(KDevQuickOpenFactory(KAboutData("kdevquickopen","kdevquickopen", ki18n("Quick Open"), "0.1", ki18n("This plugin allows quick access to project files and language-items like classes/functions."), KAboutData::License_GPL)))
 
 Declaration* cursorDeclaration() {
   IDocument* doc = ICore::self()->documentController()->activeDocument();
@@ -273,7 +277,6 @@ void QuickOpenWidget::showStandardButtons(bool show)
 
 QuickOpenWidget::QuickOpenWidget( QString title, QuickOpenModel* model, const QStringList& initialItems, const QStringList& initialScopes, bool listOnly, bool noSearchField ) : m_model(model), m_expandedTemporary(false) {
   m_filterTimer.setSingleShot(true);
-  m_filterTimer.setInterval(150);
   connect(&m_filterTimer, SIGNAL(timeout()), this, SLOT(applyFilter()));
 
   Q_UNUSED( title );
@@ -347,10 +350,28 @@ QuickOpenWidget::QuickOpenWidget( QString title, QuickOpenModel* model, const QS
   connect(o.cancelButton, SIGNAL(clicked(bool)), SIGNAL(ready()));
   
   updateProviders();
+  updateTimerInterval(true);
+  
 // no need to call this, it's done by updateProviders already
 //   m_model->restart();
 }
 
+void QuickOpenWidget::updateTimerInterval(bool cheapFilterChange)
+{
+  const int MAX_ITEMS = 10000;
+  if ( cheapFilterChange && m_model->rowCount(QModelIndex()) < MAX_ITEMS ) {
+    // cheap change and there are currently just a few items,
+    // so apply filter instantly
+    m_filterTimer.setInterval(0);
+  } else if ( m_model->unfilteredRowCount() < MAX_ITEMS ) {
+    // not a cheap change, but there are generally
+    // just a few items in the list: apply filter instantly
+    m_filterTimer.setInterval(0);
+  } else {
+    // otherwise use a timer to prevent sluggishness while typing
+    m_filterTimer.setInterval(300);
+  }
+}
 
 void QuickOpenWidget::showEvent(QShowEvent* e)
 {
@@ -399,8 +420,6 @@ void QuickOpenWidget::prepareShow()
   
   connect( o.list->selectionModel(), SIGNAL(currentRowChanged(QModelIndex,QModelIndex)), this, SLOT(currentChanged(QModelIndex,QModelIndex)) );
   connect( o.list->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(currentChanged(QItemSelection,QItemSelection)) );
-  
-  updateScrollBarState();
 }
 
 void QuickOpenWidgetDialog::run() {
@@ -485,16 +504,10 @@ void QuickOpenWidget::updateProviders() {
   m_model->enableProviders( checkedItems, checkedScopes );
 }
 
-void QuickOpenWidget::updateScrollBarState()
-{
-  if(m_model->rowCount(QModelIndex()) > rowCountForDisablingScrollBar)
-    o.list->verticalScrollBar()->setEnabled(false);
-  else
-    o.list->verticalScrollBar()->setEnabled(true);
-}
-
 void QuickOpenWidget::textChanged( const QString& str )
 {
+  // "cheap" when something was just appended to the current filter
+  updateTimerInterval(str.startsWith(m_filter));
   m_filter = str;
   m_filterTimer.start();
 }
@@ -503,8 +516,6 @@ void QuickOpenWidget::applyFilter()
 {
   m_model->textChanged( m_filter );
 
-  updateScrollBarState();
-
   QModelIndex currentIndex = m_model->index(0, 0, QModelIndex());
   o.list->selectionModel()->setCurrentIndex( currentIndex, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows | QItemSelectionModel::Current );
 
@@ -752,6 +763,12 @@ void QuickOpenPlugin::createActionsForMainWindow(Sublime::MainWindow* /*window*/
     quickOpenFunction->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_M );
     connect(quickOpenFunction, SIGNAL(triggered(bool)), this, SLOT(quickOpenFunction()));
 
+    KAction* quickOpenDocumentation = actions.addAction("quick_open_documentation");
+    quickOpenDocumentation->setText( i18n("Quick Open &Documentation") );
+    quickOpenDocumentation->setIcon( KIcon("quickopen-documentation") );
+    quickOpenDocumentation->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_D );
+    connect(quickOpenDocumentation, SIGNAL(triggered(bool)), this, SLOT(quickOpenDocumentation()));
+
     m_quickOpenDeclaration = actions.addAction("quick_open_jump_declaration");
     m_quickOpenDeclaration->setText( i18n("Jump to Declaration") );
     m_quickOpenDeclaration->setIcon( KIcon("go-jump-declaration" ) );
@@ -823,6 +840,13 @@ QuickOpenPlugin::QuickOpenPlugin(QObject *parent,
       items << ProjectItemDataProvider::supportedItemTypes();
       m_model->registerProvider( scopes, items, m_projectItemData );
     }
+    {
+      m_documentationItemData = new DocumentationQuickOpenProvider;
+      QStringList scopes, items;
+      scopes << i18n("Includes");
+      items << i18n("Documentation");
+      m_model->registerProvider( scopes, items, m_documentationItemData );
+    }
 }
 
 QuickOpenPlugin::~QuickOpenPlugin()
@@ -833,6 +857,7 @@ QuickOpenPlugin::~QuickOpenPlugin()
   delete m_projectFileData;
   delete m_projectItemData;
   delete m_openFilesData;
+  delete m_documentationItemData;
 }
 
 void QuickOpenPlugin::unload()
@@ -968,6 +993,11 @@ void QuickOpenPlugin::quickOpenClass()
   showQuickOpen( Classes );
 }
 
+void QuickOpenPlugin::quickOpenDocumentation()
+{
+  showQuickOpenWidget(QStringList(i18n("Documentation")), QStringList(i18n("Includes")), true);
+}
+
 QSet<KDevelop::IndexedString> QuickOpenPlugin::fileSet() const {
   return m_model->fileSet();
 }
@@ -1450,8 +1480,12 @@ void QuickOpenLineEdit::focusInEvent(QFocusEvent* ev) {
     QRect widgetGeometry = QRect(mapToGlobal(QPoint(0, height())), mapToGlobal(QPoint(width(), height() + 400)));
     widgetGeometry.setWidth(700); ///@todo Waste less space
     QRect screenGeom = QApplication::desktop()->screenGeometry(this);
-    if (widgetGeometry.right() > screenGeom.right())
-        widgetGeometry.moveRight(screenGeom.right());
+    if (widgetGeometry.right() > screenGeom.right()) {
+      widgetGeometry.moveRight(screenGeom.right());
+    }
+    if (widgetGeometry.bottom() > screenGeom.bottom()) {
+      widgetGeometry.moveBottom(mapToGlobal(QPoint(0, 0)).y());
+    }
     m_widget->setGeometry(widgetGeometry);
     m_widget->show();
     
@@ -1469,6 +1503,7 @@ void QuickOpenLineEdit::hideEvent(QHideEvent* ev)
 bool QuickOpenLineEdit::eventFilter(QObject* obj, QEvent* e) {
     if (!m_widget)
         return false;
+
     switch (e->type()) {
     case QEvent::KeyPress:
     case QEvent::ShortcutOverride:
@@ -1482,7 +1517,18 @@ bool QuickOpenLineEdit::eventFilter(QObject* obj, QEvent* e) {
     case QEvent::WindowDeactivate:
         kDebug() << "closing because of window activation";
         deactivate();
-    break;
+        break;
+    // handle bug 260657 - "Outline menu doesn't follow main window on its move"
+    case QEvent::Move: {
+            QWidget* widget = qobject_cast<QWidget*>(obj);
+            Q_ASSERT(widget);
+            // close the outline menu in case a parent widget moved
+            if (widget->isAncestorOf(this)) {
+              kDebug() << "closing because of parent widget move";
+              deactivate();
+            }
+            break;
+        }
     case QEvent::FocusIn:
         if (dynamic_cast<QWidget*>(obj)) {
             QFocusEvent* focusEvent = dynamic_cast<QFocusEvent*>(e);
