@@ -128,8 +128,9 @@ QIcon Document::statusIcon() const
     return d->statusIcon;
 }
 
-bool Document::closeDocument()
+void Document::closeViews()
 {
+    kDebug() << "closing all views for the document";
     foreach (Sublime::Area *area, controller()->allAreas())
     {
         QList<Sublime::View*> areaViews = area->views();
@@ -140,10 +141,37 @@ bool Document::closeDocument()
             }
         }
     }
+    Q_ASSERT(views().isEmpty());
+}
+
+bool Document::askForCloseFeedback()
+{
+   return true;
+}
+
+bool Document::closeDocument(bool silent)
+{
+    if( !silent && !askForCloseFeedback() )
+        return false;
+    closeViews();
     deleteLater();
     return true;
 }
 
+QIcon Document::icon() const
+{
+    QIcon ret = statusIcon();
+    if (!ret.isNull()) {
+        return ret;
+    }
+    return defaultIcon();
+}
+
+QIcon Document::defaultIcon() const
+{
+    return QIcon();
+}
+
 }
 
 #include "document.moc"
